#include <iostream>

using namespace std;

template <class ContentType>
class LList;

template <class ContentType>
class LListNode
{
	public:
		ContentType vContent;
		LListNode *vNext;

		LListNode( const ContentType &oContent, LListNode<ContentType> *pNext = NULL ) : vContent(oContent), vNext(pNext)
		{}
		LListNode<ContentType> *getNext() //Joro: niama nujda ot <ContentType> tuk
		{
			return this->vNext;
		}
};

template <class ContentType>
class Bag
{
	protected:
		LListNode<ContentType> *vFirst;
	
		LListNode<ContentType> * getLastNode() const
		{
			if( this->vLength == 0 )
				return NULL;
			LListNode<ContentType> *pCurrent = this->vFirst;
			while( pCurrent->vNext )
				pCurrent = pCurrent->vNext;
			return pCurrent;
		}
	public:
		int vLength;
		Bag() : vLength(0), vFirst(NULL)
		{}
		~Bag()
		{
			LListNode<ContentType> *pTMP;
			while( this->vFirst )
			{
				pTMP = this->vFirst->getNext();
				delete this->vFirst;
				this->vFirst = pTMP;
			}
		}
		Bag( const Bag<ContentType> &oList ) : vLength(0), vFirst(NULL)
		{
			LListNode<ContentType> *pCurrent = oList.vFirst;
			while( pCurrent )
			{
				this->Add( pCurrent->vContent );
				pCurrent = pCurrent->vNext;
			}
		}
		Bag<ContentType> & operator = ( const Bag<ContentType> &oList )
		{
			if( &oList == this )
				return *this;
			this->~Bag(); //Joro: povtariam, che tova e grozno i ne v duha na OOP! ;P
			this->vLength = 0;
			this->vFirst = NULL;
			LListNode<ContentType> *pCurrent = oList.vFirst;
			while( pCurrent )
			{
				this->Add( pCurrent->vContent );
				pCurrent = pCurrent->vNext;
			}

			//Joro: niama return *this
		}
		virtual void Add( const ContentType &oContent )
		{
			if( this->vLength == 0 )
				this->vFirst = new LListNode<ContentType>( oContent );
			else
				this->getLastNode()->vNext = new LListNode<ContentType>( oContent );

			this->vLength += 1;
		}

		void print() const
		{
			LListNode<ContentType> *pCurrent = this->vFirst;
			while( pCurrent )
			{
				cout << pCurrent->vContent << " ";
				pCurrent = pCurrent->vNext;
			}
			cout << endl;
		}

		int operator -=( const ContentType &oContent )
		{
			if( this->vLength == 0 )
				return false;
			LListNode<ContentType> *pCurrent = this->vFirst;
			LListNode<ContentType> *pNext = this->vFirst->vNext;
			if( pCurrent->vContent == oContent )
			{
				delete this->vFirst;
				this->vFirst = pNext;
				this->vLength -= 1;
				return true;
			}
			
			while( pCurrent->vNext && pCurrent->vNext->vContent != oContent )
				pCurrent = pCurrent->vNext;
			if( pCurrent->vNext == NULL )
				return false;
			pNext = pCurrent->vNext->vNext;
			delete pCurrent->vNext;
			pCurrent->vNext = pNext;
			this->vLength -= 1;
			return true;
		}

		int exist( const ContentType &oContent ) const
		{
			if( this->vLength == 0 )
				return false;
			LListNode<ContentType> *pCurrent = this->vFirst;
			while( pCurrent )
			{
				if( pCurrent->vContent == oContent )
					return true;
				pCurrent = pCurrent->vNext;
			}
			return false;
		}

		virtual Bag<ContentType> operator /( const Bag<ContentType>& oList )
		{
			Bag<ContentType> oNewList(oList);
			LListNode<ContentType> *pCurrent = this->vFirst;
			while( pCurrent )
			{
				oNewList -= pCurrent->vContent;
				pCurrent = pCurrent->vNext;
			}
			return oNewList;
		}

		Bag<ContentType> operator | (const Bag<ContentType>& oList )
		{
			Bag<ContentType> oNewList(oList);
			LListNode<ContentType> *pCurrent = this->vFirst;
			while( pCurrent )
			{
				oNewList.Add(pCurrent->vContent);
				pCurrent = pCurrent->vNext;
			}
			return oNewList;
		}
};

template <class ContentType>
class Set : public Bag<ContentType>
{
	public:
		void Add( const ContentType &oContent )
		{
			if( this->exist(oContent) == false ) //Joro: zashto this??
				Bag::Add(oContent);
		}

		Set<ContentType> operator | ( const Set<ContentType> & oSet )
		{
			Set<ContentType> oNewSet(oSet);
			LListNode<ContentType> *pCurrent = this->vFirst;
			while( pCurrent )
			{
				if( oNewSet.exist(pCurrent->vContent) == false )
					oNewSet.Add(pCurrent->vContent);
				pCurrent = pCurrent->vNext;
			}
			return oNewSet;
		}
};

int main( int argv, char **argc )
{
	Bag<int> Bag1;
	Bag1.Add(1);
	Bag1.Add(2);
	Bag1.Add(3);
	Bag1.Add(5);
	Bag1.Add(4);
	Bag1 -= 5;
	cout << "Bag1: "; Bag1.print();
	Bag<int> Bag2 = Bag1;
	Bag2 -= 1;
	Bag2.Add(7);
	cout << "Bag2: "; Bag2.print();
	cout << "Bag1|Bag2: "; (Bag1|Bag2).print();
	cout << "Bag1/Bag2: ";(Bag1/Bag2).print();

	cout << endl << endl << "Set examples " << endl;
	Set<int> oSet1;
	oSet1.Add(1);
	oSet1.Add(3);
	oSet1.Add(3);
	oSet1.Add(3);
	oSet1.Add(-9);
	cout << "Set1: "; oSet1.print();
	Set<int> oSet2(oSet1);
	oSet2.Add(7);
	cout << "Set2: "; oSet2.print();
	cout << "Set1|Set2: "; (oSet1|oSet2).print();
	cout << "Set1/Set2: ";(oSet1/oSet2).print();

	system("PAUSE");
	return 0;
}
