#include <iostream>
#include <math.h>
#include <fstream>

using namespace std;

class Point
{
	public:
		double X, Y;
		Point( double iX=0, double iY=0 ) : X(iX), Y(iY)
		{}
		double getDistTo( const Point& oPoint ) const
		{
			return sqrt( (oPoint.X - this->X)*(oPoint.X - this->X) + (oPoint.Y - this->Y)*(oPoint.Y - this->Y) );
		}
		void translate( int iX, int iY )
		{
			this->X += iX;
			this->Y += iY;
		}
		void print() const
		{
			cout << "(" << this->X << "," << this->Y << ")" << endl;
		}

		void serialize( ofstream &oStream )
		{
			oStream << (this->X) << (this->Y) << endl;
		}
};

class Parallelogram
{

	protected:
		Point vVertexes[4];
	public:
		virtual double getArea() = NULL;
		virtual double getP() = NULL;
		virtual void translate( int, int ) = NULL;
		virtual void print() = NULL;
		virtual void serialize( ofstream & ) = NULL;
		virtual int getType() = NULL;
};

class Diamond : public virtual Parallelogram
{
		
	public:
		Diamond( const Point &A, const Point &B, const Point &C, const Point &D )
		{
			this->vVertexes[0] = A;
			this->vVertexes[1] = B;
			this->vVertexes[2] = C;
			this->vVertexes[3] = D;
		}

		double getArea()
		{
			cout << "calc area of Diamond" << endl; //Joro: kultovo :D
			return rand();
		}

		double getP()
		{
			cout << "calc p of Diamond" << endl;
			return rand();
		}

		void translate( int iX, int iY )
		{
			for( int i=0; i != 4; ++i )
				this->vVertexes[i].translate(iX,iY);
		}

		void print()
		{
			cout << "**Diamond**" << endl;
			for( int i=0; i != 4; ++i )
				(this->vVertexes[i]).print();
		}

		void serialize( ofstream &oStream )
		{
			for( int i=0; i != 4; ++i )
				this->vVertexes[i].serialize(oStream);
		}

		int getType()
		{
			return 1;
		}
};

class Rectangle : public virtual Parallelogram
{
	public:
		Rectangle( const Point &A, const Point &B, const Point &C, const Point &D )
		{
			this->vVertexes[0] = A;
			this->vVertexes[1] = B;
			this->vVertexes[2] = C;
			this->vVertexes[3] = D;
		}

		double getArea()
		{
			cout << "calc area of Rectangle" << endl;
			return rand();
		}

		double getP()
		{
			cout << "calc p of Rectangle" << endl;
			return rand();
		}

		void translate( int iX, int iY )
		{
			for( int i=0; i != 4; ++i )
				this->vVertexes[i].translate(iX,iY);
		}

		void print()
		{
			cout << "**Rectangle**" << endl;
			for( int i=0; i != 4; ++i )
				(this->vVertexes[i]).print();
		}

		void serialize( ofstream &oStream )
		{
			for( int i=0; i != 4; ++i )
				this->vVertexes[i].serialize(oStream);
		}

		int getType()
		{
			return 2;
		}
};

class Square : public Rectangle, public Diamond
{
	public:
		Square( const Point &A, const Point &B, const Point &C, const Point &D ) : Rectangle(A, B, C, D), Diamond(A, B, C, D)
		{}

		double getArea()
		{
			cout << "calc area of Square" << endl;
			return rand();
		}

		double getP()
		{
			cout << "calc p of Square" << endl;
			return rand();
		}

		void translate( int iX, int iY )
		{
			for( int i=0; i != 4; ++i )
				this->vVertexes[i].translate(iX,iY);
		}

		void print()
		{
			cout << "**Square**" << endl;
			for( int i=0; i != 4; ++i )
				(this->vVertexes[i]).print();
		}

		void serialize( ofstream &oStream )
		{
			for( int i=0; i != 4; ++i )
				this->vVertexes[i].serialize(oStream);
		}
		int getType()
		{
			return 3;
		}
};

class Shapes
{
		Parallelogram **vContent;
		int vShapeCount;
	public:
		Shapes():vContent(NULL), vShapeCount(0)
		{
		}
		void AddShape()
		{
			Parallelogram *pNewShape;
			cout << "Type in shape(1,2 or 3): ";
			int iShapeType;
			cin >> iShapeType;
			Point aPoints[4];
			cout << "Type in four points: " << endl;
			/*for( int i=0; i != 4; ++i )
			{
				cout << "Type in (X Y): ";
				cin >> aPoints[i].X >> aPoints[i].Y;
			}*/
			switch(iShapeType)
			{
				case 1:
					pNewShape = new Diamond( aPoints[0], aPoints[1], aPoints[2], aPoints[3] );
					break;
				case 2:
					pNewShape = new Rectangle( aPoints[0], aPoints[1], aPoints[2], aPoints[3] );
					break;
				case 3:
					pNewShape = new Square( aPoints[0], aPoints[1], aPoints[2], aPoints[3] );
					break;
				default:
					cout << "Unknown figure!" << endl;
					return;
			}

			Parallelogram **aNewContent = new Parallelogram*[this->vShapeCount + 1];
			int i;
			for( i=0; i != this->vShapeCount; ++i )
				aNewContent[i] = this->vContent[i];
			aNewContent[i] = pNewShape;
			delete this->vContent;
			this->vContent = aNewContent;
			this->vShapeCount += 1;
		}

		void printShapes()
		{
			int i;
			for( i=0; i != this->vShapeCount; ++i )
				this->vContent[i]->print();
		}

		void translateAll( int x, int y )
		{
			int i;
			for( i=0; i != this->vShapeCount; ++i )
				this->vContent[i]->translate(x, y);
		}

		void SaveToFile( char *sFileName )
		{

			ofstream oStream(sFileName);
			oStream << this->vShapeCount << endl;
			for( int i=0; i != this->vShapeCount; ++i )
			{
				oStream << this->vContent[i]->getType() << endl;
				this->vContent[i]->serialize(oStream);
			}
		}
};

int main( int argv, char **argc )
{

	Shapes oShapes;
	oShapes.AddShape();
	oShapes.AddShape();
	oShapes.AddShape();
	oShapes.AddShape();
	oShapes.AddShape();
	oShapes.translateAll(1,1);
	oShapes.printShapes();

	oShapes.SaveToFile("shapes.txt");

	system("PAUSE");
	return 0;
}